# -*- coding: utf-8 -*-
"""
Created on Thu Aug 22 11:12:10 2019

scripts that reads the locations of the lander frames and deployment periods per cruise.
it also converts the lat/lon coordinates to ETRS89 UTM31N x,y pairs.
results are stored in csv files with format:
    yyyy_mm_'LOCATION'_'LOCCODE'_frames.csv

$Id: casino2csv_frames.py 37 2019-08-26 09:20:43Z erikhendriks $
$Date: 2019-08-26 11:20:43 +0200 (ma, 26 aug 2019) $
$Author: erikhendriks $
$Revision: 37 $
$HeadURL: https://svn.citg.tudelft.nl/sandbox/trunk/cruises/scripts/casino2csv_frames.py $

@author: 
    erikhendriks
"""

#import packages

from pathlib import Path
import pandas as pd
from pyproj import Proj, transform
import os
import sys
from datetime import datetime as dt
import numpy as np

# lander names
lst = ['ERIKS','TUD','EDDY', 'MOOR', 'EGMOND','EGM','ALTRAP']
pat = '|'.join(lst)
inProj = Proj(init='epsg:4326')
outProj = Proj(init='epsg:25831')

# define directories
scriptsDir = Path(os.path.abspath(sys.argv[0]))
baseDir = scriptsDir.parents[2] #for standard layout of repository, this gives \\trunk directory
rawDir = baseDir / 'cruises' / 'raw' 
campaignDirs = [x for x in rawDir.glob('*/')] # find all directories in this directory
campaignCode=[str(x).split(sep=os.sep)[-1] for x in campaignDirs] # this includes: year_month_location_code
outputDir=baseDir / 'cruises' / 'meta' 

for idx,campaignDir in enumerate(campaignDirs):
    
    # load CASINO file
    clist=[x for x in campaignDir.glob('CASINO*.csv')]
    if '2019_05_egmond_sbvn' not in str(campaignDir):
        casino=pd.read_csv(clist[0],delimiter='\t',encoding = "ISO-8859-1",low_memory=False)
    else:
        casino=pd.read_csv(clist[0],delimiter=',',encoding = "ISO-8859-1")
        # get the 'operations' from casino
    operations=casino.loc[casino['Type']=='OPE',:]
    operations.loc[:,'DEVICE CODE']= operations.loc[:,'Device code'].str.upper()
    #select the operations where the device that was deployed matches the device list as indicated before
    deploy=operations[operations['DEVICE CODE'].str.contains(pat)]
    deploy.loc[:,'datetime']=pd.to_datetime(deploy['Date'] +' '+ deploy['Heure'],utc=True)
    
    # replace strings if needed
    deploy.loc[:,'DEVICE CODE']=deploy.replace({'DEVICE CODE' : { 'ERIKS' : 'TUD', 'EGMOND' : 'EGM', 'ALTRAP' : 'EGM', 'MOORING' : 'MOOR' }})
    
    # create dataframe that will be output to csv

    
    
    # first obtain latitude & longitude of frame - determined where deployment took place
    # then save the dataframe to a csv file
    if len(deploy)==2*len(deploy['DEVICE CODE'].unique()): #if-statement needed because of double deployments in 2017_10_egmond_sbns campaign
        output=pd.DataFrame(index=deploy['DEVICE CODE'].unique(),columns=['lat','lon','ETRS89_UTM31N_x','ETRS89_UTM31N_y','date_begin','date_end'])
        output.index.name = 'frame'
        for frame in deploy['DEVICE CODE'].unique():
            output.loc[frame,'lat']=deploy.loc[(deploy['Action name'].str.lower()=='deployment') & (deploy['DEVICE CODE']==frame),'Latitude (degres, minutes)'].values[0]
            output.loc[frame,'lon']=deploy.loc[(deploy['Action name'].str.lower()=='deployment') & (deploy['DEVICE CODE']==frame),'Longitude (degres, minutes)'].values[0]
            output.loc[frame,'date_begin']=deploy.loc[(deploy['Action name'].str.lower()=='deployment') & (deploy['DEVICE CODE']==frame),'datetime'].values[0]
            output.loc[frame,'date_end']=deploy.loc[(deploy['Action name'].str.lower()=='recovery') & (deploy['DEVICE CODE']==frame),'datetime'].values[0]
    else:
        output=pd.DataFrame(index=deploy['DEVICE CODE'].unique(),
                            columns=['lat','lon','ETRS89_UTM31N_x','ETRS89_UTM31N_y','date_begin','date_end','date_begin2','date_end2'])
        output.index.name = 'frame'
        for frame in deploy['DEVICE CODE'].unique():
            output.loc[frame,'lat']=deploy.loc[(deploy['Action name'].str.lower()=='deployment') & (deploy['DEVICE CODE']==frame),'Latitude (degres, minutes)'].values[0]
            output.loc[frame,'lon']=deploy.loc[(deploy['Action name'].str.lower()=='deployment') & (deploy['DEVICE CODE']==frame),'Longitude (degres, minutes)'].values[0]
            output.loc[frame,'date_begin']=deploy.loc[(deploy['Action name'].str.lower()=='deployment') & (deploy['DEVICE CODE']==frame),'datetime'].values[0]
            output.loc[frame,'date_end']=deploy.loc[(deploy['Action name'].str.lower()=='recovery') & (deploy['DEVICE CODE']==frame),'datetime'].values[0]        
            if frame=='EGM':
                output.loc[frame,'date_begin2']=deploy.loc[(deploy['Action name'].str.lower()=='deployment') & (deploy['DEVICE CODE']==frame),'datetime'].values[1]
                output.loc[frame,'date_end2']=deploy.loc[(deploy['Action name'].str.lower()=='recovery') & (deploy['DEVICE CODE']==frame),'datetime'].values[1]      
                output.loc[:,'date_begin2']=pd.to_datetime(output['date_begin2'],utc=True) 
                output.loc[:,'date_end2']=pd.to_datetime(output['date_end2'],utc=True) 
    # convert date_begin and date_end to datetime64[ns] format
    output.loc[:,'date_begin']=pd.to_datetime(output['date_begin'],utc=True) 
    output.loc[:,'date_end']=pd.to_datetime(output['date_end'],utc=True) 
    # as a latest step, transform the lat, lon pair to x,y coordinates
    output.loc[:,'ETRS89_UTM31N_x'],output.loc[:,'ETRS89_UTM31N_y'] = transform(inProj,outProj,output.loc[:,'lon'].values,output.loc[:,'lat'].values)    
    
    output.to_csv(outputDir / str(campaignCode[idx]+'_locations.csv'))





